var config = new Config();
var store;

var testcase_items = new Array();
var active = false;
var empty = true;
var tabId = null;

console.log('init');

config.loadConfiguration(function() {
    console.log("initial configuration", config);
    store = new Store(config);
});

chrome.storage.onChanged.addListener(function() {
    config.loadConfiguration(function() {
        console.log("configuration changed", config);
        store = new Store(config);
    });
});

var sendToActiveTab = function(request, callback) {
    chrome.tabs.query({
        active: true,
        currentWindow: true
    }, function(tabs) {
        if (tabs.length < 1) {
            this.console.log("couldn't find active tab");
        }
        else {
            var tab = tabs[0];
            chrome.tabs.sendMessage(tab.id, request, callback);
        }
    });
};

chrome.runtime.onMessage.addListener(
        function(request, sender, sendResponse) {
            console.log("chrome.runtime.onMessage", request);
            if (request.action == "append") {
                testcase_items[testcase_items.length] = request.obj;
                empty = false;
                sendResponse({});
                connections[tabId].postMessage({item: request.obj, id: testcase_items.length});
                return true;
            }
            if (request.action == "poke") {
                testcase_items[testcase_items.length - 1] = request.obj;
                sendResponse({});
                return true;
            }
            if (request.action == "get_status") {
                sendResponse({'active': active, 'empty': empty});
                return true;
            }
            if (request.action == "start") {
                if (!active) {
                    active = true;
                    empty = true;
                    testcase_items = new Array();
                    var url = null;
                    chrome.tabs.query({
                        active: true,
                        currentWindow: true
                    }, function(tabs) {
                        if (tabs.length < 1) {
                            this.console.log("couldn't find active tab");
                        }
                        else {
                            var tab = tabs[0];
                            tabId = tab.id;
                            chrome.tabs.update(tab.id, {url: tab.url}, function(tab) {
                                chrome.tabs.sendMessage(tabId, {action: "open"});
                                sendResponse({start: true});
                            });
                        }
                    });

                }
                return true;
            }
            if (request.action == "stop") {
                active = false;
                chrome.tabs.sendMessage(tabId, {action: "stop"});
                request.sitemap.recorder = testcase_items;
                connections[tabId].postMessage({sitemap: request.sitemap});
                return true;
            }
            if (request.action == "get_items") {
                sendResponse({'items': testcase_items});
                return true;
            }

            if (request.createSitemap) {
                store.createSitemap(request.sitemap, sendResponse);
                return true;
            }
            else if (request.saveSitemap) {
                store.saveSitemap(request.sitemap, sendResponse);
                return true;
            }
            else if (request.deleteSitemap) {
                store.deleteSitemap(request.sitemap, sendResponse);
                return true;
            }
            else if (request.getAllSitemaps) {
                store.getAllSitemaps(sendResponse);
                return true;
            }
            else if (request.sitemapExists) {
                store.sitemapExists(request.sitemap, sendResponse);
                return true;
            }
            else if (request.getSitemapData) {
                store.getSitemapData(new Sitemap(request.sitemap), sendResponse);
                return true;
            }
            else if (request.saveLng) {
                chrome.storage.local.set({'lng' : request.lng});
                sendResponse({'return':true});
                return true;
            }
            else if (request.getLng) {
                chrome.storage.local.get('lng', function (items){
                    sendResponse(items);
                });
                return true;
            }
            else if (request.selectSelector) {

                sendToActiveTab(request, function(response) {
                    console.log("selectors selected", response);
                    sendResponse(response);
                });
                return true;
            }
            else if (request.changeSelector) {
                sendToActiveTab(request);

                return true;
            }
            else if (request.selectSelectorParent || request.selectSelectorChild) {

                sendToActiveTab(request, function(response) {
                    sendResponse(response);
                });
                return true;
            }
            else if (request.previewSelector || request.cancelPreviewSelector) {
                chrome.tabs.query({
                    active: true,
                    currentWindow: true
                }, function(tabs) {
                    if (tabs.length < 1) {
                        this.console.log("couldn't find active tab");
                    }
                    else {
                        var tab = tabs[0];
                        chrome.tabs.sendMessage(tab.id, request);
                    }
                });
                return false;
            }
            else if (request.scrapeSitemap) {
                var sitemap = new Sitemap(request.sitemap);
                var queue = new Queue();
                var browser = new ChromePopupBrowser();

                var scraper = new Scraper({
                    queue: queue,
                    sitemap: sitemap,
                    browser: browser,
                    store: store
                });

                try {
                    scraper.run(function() {
                        browser.close();
                        if (window.webkitNotifications.checkPermission() == 0) {
                            var notification = webkitNotifications.createNotification(
                                    'assets/images/icon38.png',
                                    'Scraping finished1',
                                    'Finished scraping ' + sitemap._id
                                    );
                            notification.show();
                        }
                    });
                }
                catch (e) {
                    console.log("Scraper execution cancelled".e);
                }
            }
            else if (request.previewSelectorData) {
                chrome.tabs.query({
                    active: true,
                    currentWindow: true
                }, function(tabs) {
                    if (tabs.length < 1) {
                        this.console.log("couldn't find active tab");
                    }
                    else {
                        var tab = tabs[0];
                        chrome.tabs.sendMessage(tab.id, request, sendResponse);
                    }
                });
                return true;
            }
        }
);

var connections = {};

chrome.runtime.onConnect.addListener(function(port) {

    var extensionListener = function(message, sender, sendResponse) {

        if (message.name == "init") {
            connections[message.tabId] = port;
            return;
        }

    }

    port.onMessage.addListener(extensionListener);

    port.onDisconnect.addListener(function(port) {
        port.onMessage.removeListener(extensionListener);

        var tabs = Object.keys(connections);
        for (var i = 0, len = tabs.length; i < len; i++) {
            if (connections[tabs[i]] == port) {
                delete connections[tabs[i]]
                break;
            }
        }
    });
});

