var timeout = 5000;
var ws;

var SitemapController = function (options) {
    for (var i in options) {
        this[i] = options[i];
    }
    this.init();
};

SitemapController.prototype = {
    control: function (controls) {
        var controller = this;

        for (var selector in controls) {
            for (var event in controls[selector]) {
                $(document).on(event, selector, (function (selector, event) {
                    return function () {
                        var continueBubbling = controls[selector][event].call(controller, this);
                        if (continueBubbling !== true) {
                            return false;
                        }
                    }
                })(selector, event));
            }
        }
    },
    /**
     * Loads templates for ICanHaz
     */
    loadTemplates: function (cbAllTemplatesLoaded) {
        var templateIds = [
            'Viewport',
            'SitemapList',
            'SitemapListItem',
            'SitemapCreate',
            'SitemapImport',
            'SitemapExport',
            'SitemapBrowseData',
            'SitemapExportDataCSV',
            'SitemapEditMetadata',
            'SelectorList',
            'SelectorListItem',
            'SelectorEdit',
            'SelectorEditTableColumn',
            'SitemapSelectorGraph',
            'DataPreview',
            'SitemapRecordSteps'
        ];
        var templatesLoaded = 0;
        var cbLoaded = function (templateId, template) {
            templatesLoaded++;
            ich.addTemplate(templateId, template);
            if (templatesLoaded === templateIds.length) {
                cbAllTemplatesLoaded();
            }
        }

        templateIds.forEach(function (templateId) {
            $.get(this.templateDir + templateId + '.html', cbLoaded.bind(this, templateId));
        }.bind(this));
    },
    init: function () {
        this.loadTemplates(function () {
            // currently viewed objects
            this.clearState();

            jQuery(document).on('showModal', function(e, messageCode, headerCode){
                jQuery('#infoModal .modal-body').html(i18n.t(messageCode));
                jQuery('#infoModal .modal-title').html(i18n.t(headerCode));
                jQuery('#infoModal').modal();
            });

            // render main viewport
            ich.Viewport().appendTo("body");

            // cancel all form submits
            $("form").bind("submit", function () {
                return false;
            });

            this.control({
                '#sitemaps-nav-button': {
                    click: this.showSitemaps
                },
                '#sitemap-record-steps-nav-button': {
                    click: this.showRecorder
                },
                '#create-sitemap-create-nav-button': {
                    click: this.showCreateSitemap
                },
                '#create-sitemap-import-nav-button': {
                    click: this.showImportSitemapPanel
                },
                '#sitemap-export-nav-button': {
                    click: this.showSitemapExportPanel
                },
                '#sitemap-export-data-csv-nav-button': {
                    click: this.showSitemapExportDataCsvPanel
                },
                '#submit-create-sitemap': {
                    click: this.createSitemap
                },
                '#submit-import-sitemap': {
                    click: this.importSitemap
                },
                '#sitemap-edit-metadata-nav-button': {
                    click: this.editSitemapMetadata
                },
                '#sitemap-selector-list-nav-button': {
                    click: this.showSitemapSelectorList
                },
                '#sitemap-selector-graph-nav-button': {
                    click: this.showSitemapSelectorGraph
                },
                '#sitemap-browse-nav-button': {
                    click: this.browseSitemapData
                },
                'button#submit-edit-sitemap': {
                    click: this.editSitemapMetadataSave
                },
                '#sitemaps tr': {
                    click: this.editSitemap
                },
                '#sitemaps button[action=delete-sitemap]': {
                    click: this.deleteSitemap
                },
                '#sitemaps button[action=recorder-start]': {
                    click: this.start
                },
                '#sitemaps button[action=recorder-stop]': {
                    click: this.stop
                },
                '#sitemaps button[recorder-export]': {
                    click: this.export
                },
                '#sitemap-scrape-nav-button': {
                    click: this.scrapeSitemap
                },
                "#sitemaps button[action=browse-sitemap-data]": {
                    click: this.sitemapListBrowseSitemapData
                },
                '#sitemaps button[action=csv-download-sitemap-data]': {
                    click: this.downloadSitemapData
                },
                '#selector-tree .breadcrumb a': {
                    click: this.treeNavigationshowSitemapSelectorList
                },
                '#selector-tree tr button[action=edit-selector]': {
                    click: this.editSelector
                },
                '#edit-selector button[action=save-selector]': {
                    click: this.saveSelector
                },
                '#edit-selector button[action=cancel-selector-editing]': {
                    click: this.cancelSelectorEditing
                },
                '.actions button[action=add-selector]': {
                    click: this.addSelector
                },
                "#selector-tree tr button[action=delete-selector]": {
                    click: this.deleteSelector
                },
                "#selector-tree tr button[action=preview-selector]": {
                    click: this.previewSelectorFromSelectorTree
                },
                "#selector-tree tr button[action=data-preview-selector]": {
                    click: this.previewSelectorDataFromSelectorTree
                },
                "#edit-selector button[action=select-selector]": {
                    click: this.selectSelector
                },
                "#edit-selector button[action=select-selector-parent]": {
                    click: this.selectSelectorParent
                },
                "#edit-selector button[action=select-selector-child]": {
                    click: this.selectSelectorChild
                },
                "#edit-selector button[action=preview-selector]": {
                    click: this.previewSelector
                },
                "#edit-selector button[action=preview-selector-data]": {
                    click: this.previewSelectorDataFromSelectorEditing
                },
                ".run-casper": {
                    click: this.runCasper
                },
                ".stop-casper": {
                    click: this.stopCasper
                },
                "a[data-lng]": {
                    click: this.changeLanguage
                },
                "#config-form button.casper-init": {
                    click: this.initCasper
                },
                "#upload-config-file": {
                    click: this.uploadConfigForm
                },
                'button[name="updateStatus"]': {
                    click: this.updateStatus
                }
            });
            this.showSitemaps(false);
        }.bind(this));
    },
    initCasper: function () {
        var server = jQuery('input[name="server"]').val();
        var username = jQuery('input[name="username"]').val();
        var password = jQuery('input[name="password"]').val();
        var configuration = jQuery('input[name="configuration"]').val();

        jQuery.ajax({
            url: "http://" + server + "/init",
            xhrFields: {
                withCredentials: true
            },
            type: 'POST',
            data: {username: username, password: password, configuration: configuration},
            success: function (result) {
                jQuery(document).trigger('showModal', ['configured', 'information']);
            },
            error: function (jqXHR, textStatus, ex) {
                jQuery(document).trigger('showModal', ['connectionFailed', 'error']);
            }
        });
    },
    updateStatus: function () {
        var $label = jQuery('.label-update-status');

        var server = jQuery('input[name="server"]').val();
        var username = jQuery('input[name="username"]').val();
        var password = jQuery('input[name="password"]').val();

        jQuery.ajax({
            url: 'http://' + server + '/status',
            xhrFields: {
                withCredentials: true
            },
            type: 'POST',
            data: {username: username, password: password},
            success: function (result) {
                $label.removeClass('label-danger').addClass('label-success');
                $label.text('Online');
                jQuery('.action-buttons').removeClass('hidden');
            },
            error: function (jqXHR, textStatus, ex) {
                $label.removeClass('label-success').addClass('label-danger');
                $label.text('Offline');
                jQuery('.action-buttons').addClass('hidden');
            }
        });
    },
    uploadConfigForm: function () {
        var formData = new FormData();
        var server = jQuery('input[name="server"]').val();
        formData.append('file', jQuery('#config')[0].files[0]);
        formData.append('configuration', jQuery('input[name="configuration"]').val());
        formData.append('username', jQuery('input[name="username"]').val());
        formData.append('password', jQuery('input[name="password"]').val());

        jQuery.ajax({
            url: 'http://' + server + '/upload-config-form',
            type: 'POST',
            data: formData,
            enctype: 'multipart/form-data',
            processData: false,  // tell jQuery not to process the data
            contentType: false,  // tell jQuery not to set contentType
            success: function (data) {
                jQuery(document).trigger('showModal', ['uploaded', 'information']);

            },
            error: function (jqXHR, textStatus, errorThrown) {
                jQuery(document).trigger('showModal', ['uploadedError', 'error']);
            }
        });
    },
    downloadResults: function () {
        var server = jQuery('input[name="server"]').val();
        var file = jQuery(this).data('file');
        var plugin = jQuery(this).data('plugin');
        var username = jQuery('input[name="username"]').val();
        var password = jQuery('input[name="password"]').val();

        jQuery.fileDownload("http://" + server + "/download?plugin=" + plugin + '&file=' + file, {
            failCallback: function () {
                jQuery(document).trigger('showModal', ['downloadError', 'error']);
            },
            httpMethod: "POST",
            data: {
                username: username,
                password: password
            }
        });
    },
    setupLanguage: function (lng) {
        this.store.saveLng(lng);
        if (lng == 'en') {
            jQuery('a[data-lng="en"]').hide();
            jQuery('a[data-lng="cs"]').show();
        } else {
            jQuery('a[data-lng="cs"]').hide();
            jQuery('a[data-lng="en"]').show();
        }

        i18n.init({lng: lng, resGetPath: '../locales/__lng__/__ns__.json'}, function () {
            jQuery("body").i18n();
        });
    },
    changeLanguage: function (button) {
        var lng = jQuery(button).data('lng');
        this.setupLanguage(lng);
        setTimeout(function () {
            jQuery(".chzn-select").trigger("chosen:updated");

            if (jQuery('#filters_chosen .search-field input').val()) {
                jQuery('#filters_chosen .search-field input').val(i18n.t('selectOptions'));
            }
        }, 3000);
    },
    stopCasper: function () {
        jQuery(".screenshot img").remove();
        jQuery('.run-casper').removeClass('hidden');
        jQuery('.stop-casper').addClass('hidden');
        ws.close();
    },
    runCasper: function () {
        jQuery('.stop-casper').removeClass('hidden');
        jQuery('.run-casper').addClass('hidden');
        jQuery('.cmd-output').empty();
        this.send();
    },
    send: function () {
        var proxyServer = jQuery('input[name="proxyServer"]').val();
        var proxyUsername = jQuery('input[name="proxyUsername"]').val();
        var proxyPassword = jQuery('input[name="proxyPassword"]').val();
        var username = jQuery('input[name="username"]').val();
        var password = jQuery('input[name="password"]').val();

        var sitemap = this.state.currentSitemap;
        var server = jQuery('input[name="server"]').val();

        var configuration = {
            'id': sitemap._id,
            'proxyServer': proxyServer,
            'proxyUsername': proxyUsername,
            'proxyPassword': proxyPassword,
            'username': username,
            'password': password
        };

        var showImage = window.setInterval(function () {
            var imageUrl = 'http://' + server + '/screenshot?' + new Date().getTime();
            jQuery(".screenshot img").remove();
            jQuery(".screenshot").attr('href', imageUrl);
            jQuery(".screenshot").append('<img class="img-responsive" src="' + imageUrl + '" alt="Screenshot" />');
        }, 2000);

        try {
            ws = new WebSocket('ws://' + server, 'echo-protocol');
        } catch (err) {
            setTimeout(startWs, timeout);
        }

        ws.onopen = function (event) {
            ws.send(JSON.stringify(configuration));
        }

        ws.onmessage = function (event) {
            jQuery('.cmd-output').prepend('<div>' + event.data + '</div>');
        };

        ws.onclose = function () {
            jQuery('.stop-casper').addClass('hidden');
            jQuery('.run-casper').removeClass('hidden');
            window.clearInterval(showImage);
        }

    },
    clearState: function () {
        this.state = {
            // sitemap that is currently open
            currentSitemap: null,
            // selector ids that are shown in the navigation
            editSitemapBreadcumbsSelectors: null,
            currentParentSelectorId: null,
            currentSelector: null
        };
    },
    start: function () {
        jQuery('.progress-img > div:first-child > div').removeClass('hidden');
        jQuery('.recorder-item').remove();
        jQuery('#bgo').addClass('hidden');
        jQuery('#bstop').removeClass('hidden');
        var backgroundPageConnection = chrome.runtime.connect({
            name: "devtool-page"
        });

        this.timer = setInterval(function () {
            jQuery('.glyphicon-record').toggleClass('hidden')
        }, 2000);

        backgroundPageConnection.onMessage.addListener(function (message) {
            if (message.sitemap) {
                jQuery('.recorder-sitemap').val(JSON.stringify(message.sitemap));
            }

            if ((message.item && message.item.type == 'text' && message.item.type == 23)
                || (message.item && message.item.type == 1) || (message.item && message.item.type == 4)) {
                jQuery('.recorder-configuration .alert').remove();

                var appendRow = '<div class="col-md-12 recorder-item">'
                    + '<div class="row">'
                    + '<div class="col-md-2">'
                    + '<input type="checkbox" data-item-id="' + message.id + '" value="1" name="countStep[]">'
                    + '<label data-i18n="countStep">'
                    + i18n.t('countStep')
                    + '</label>'
                    + '</div>'
                    + '<div class="col-md-4">'
                    + '<input type="text" class="form-control col-md-5 delay" data-i18n="[placeholder]formInputDelay" '
                    + 'name="delay[]" data-item-id="' + message.id + '" placeholder="' + i18n.t('formInputDelay') + '" value="" />'
                    + '</div>'
                    + '<div class="col-md-4">'
                    + '<input type="text" class="form-control col-md-5 column" data-i18n="[placeholder]formInputName" '
                    + 'data-item-id="' + message.id + '" name="column[]" value="" placeholder="' + i18n.t('formInputName') + '"/>'
                    + '</div>'
                    + '<div class="col-md-2">'
                    + '<button class="btn btn-danger btn-sm form-configuration-remove"><span class="glyphicon glyphicon-remove"></span></button>'
                    + '</div>'
                    + '</div>'
                    + '</div>';
                jQuery('.recorder-configuration').append(
                    appendRow
                );
                jQuery(".column").on("keyup", function(){
                    jQuery(this).val(jQuery(this).val().toString().replace(/[^a-zA-Z0-9\-]/g,'_'));
                })
                jQuery('.form-configuration-remove').on('click', function () {
                    jQuery(this).parent().parent().remove();
                    if (!jQuery('.recorder-item').length) {
                        this.showNoItemsMessage('.recorder-configuration', 'noItemsConfigForm');
                    }
                });

                if (!jQuery('.recorder-item').length) {
                    this.showNoItemsMessage('.recorder-configuration', 'noItemsConfigForm');
                }

                if ((message.item.type == 1 && message.item.info.type != 'text') || message.item.type == 4) {
                    jQuery('.columns[data-item-id="' + message.id + '"]').addClass('hidden');
                }
            }


        });
        backgroundPageConnection.postMessage({
            name: 'init',
            tabId: chrome.devtools.inspectedWindow.tabId
        });
        chrome.runtime.sendMessage({action: "start", sitemap: this.state.currentSitemap});
    },
    stop: function () {
        jQuery('.progress-img > div:first-child > div').addClass('hidden');
        jQuery('#bstop').addClass('hidden');
        jQuery('#bgo').removeClass('hidden');
        clearInterval(this.timer);
        jQuery('.glyphicon-record').addClass('hidden');
        chrome.runtime.sendMessage({
            action: "stop",
            sitemap: this.state.currentSitemap,
            tabId: chrome.devtools.inspectedWindow.tabId
        });

    },
    setStateEditSitemap: function (sitemap) {
        this.state.currentSitemap = sitemap;
        this.state.editSitemapBreadcumbsSelectors = [
            {id: '_root'}
        ];
        this.state.currentParentSelectorId = '_root';
    },
    showCreateSitemap: function () {
        var sitemapForm = ich.SitemapCreate();
        $("#viewport").html(sitemapForm);
        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });
        return true;
    },
    showRecorder: function () {
        var sitemapForm = ich.SitemapRecordSteps();
        $("#viewport").html(sitemapForm);

        if (this.state.currentSitemap.formConfiguration) {
            for (var i = 0; i < this.state.currentSitemap.formConfiguration.length; i++) {
                jQuery('.recorder-configuration .alert').remove();
                var row = '<div class="col-md-12 recorder-item">'
                    + '<div class="row">'
                    + '<div class="col-md-2">'
                    + '   <input type="checkbox" '
                    + '          value="1" '
                    + '          data-item-id="' + this.state.currentSitemap.countStep[i].id + '"'
                    + (this.state.currentSitemap.countStep[i].value ? 'checked="checked"' : '')
                    + '          name="countStep[]" >'
                    + '   <label data-i18n="countStep">'
                    + '      Count step'
                    + '   </label>'
                    + '</div>'
                    + '<div class="col-md-4">'
                    + '   <input type="text" class="form-control col-md-5 delay"'
                    + '          data-i18n="[placeholder]formInputDelay"'
                    + '          name="delay[]"'
                    + '          data-item-id="' + this.state.currentSitemap.delay[i].id + '"'
                    + '          placeholder="Delay (time in seconds)" '
                    + '          value="' + this.state.currentSitemap.delay[i].value + '" />'
                    + '   </div>'
                    + '   <div class="col-md-4">'
                    + '   <input type="text" class="form-control col-md-5 column"'
                    + '          data-i18n="[placeholder]formInputName"'
                    + '          name="column[]"'
                    + '          data-item-id="' + this.state.currentSitemap.formConfiguration[i].id + '"'
                    + '          value="' + this.state.currentSitemap.formConfiguration[i].value + '"'
                    + '          placeholder="Input name"/>'
                    + '   </div>'
                    + '   <div class="col-md-2">'
                    + '   <button class="btn btn-danger btn-sm form-configuration-remove"><span class="glyphicon glyphicon-remove"></span></button>'
                    + '   </div>'
                    + '   </div>'
                    + '   </div>';

                jQuery('.recorder-configuration').append(row);
                    jQuery(".column").on("keyup", function(){
                        jQuery(this).val(jQuery(this).val().toString().replace(/[^a-zA-Z0-9\-]/g,'_'));
                    })
            }
        }

        jQuery('.form-configuration-remove').on('click', function () {
            jQuery(this).closest('.col-md-12').remove();
            if (!jQuery('.recorder-item').length) {
                this.showNoItemsMessage('.recorder-configuration', 'noItemsConfigForm');
            }
        });

        if (!jQuery('.recorder-item').length) {
            this.showNoItemsMessage('.recorder-configuration', 'noItemsConfigForm');
        }

        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });
        return true;
    },
    showNoItemsMessage: function (selector, message) {
        jQuery(selector).append(
            '<div class="alert alert-info" role="alert" data-i18n="' + message + '">No items</div>'
        );
    },
    showImportSitemapPanel: function () {
        var sitemapForm = ich.SitemapImport();
        $("#viewport").html(sitemapForm);
        return true;
    },
    showSitemapExportPanel: function () {
        var sitemap = this.state.currentSitemap;
        var sitemapJSON = sitemap.exportSitemap();
        var sitemapExportForm = ich.SitemapExport({
            sitemapJSON: sitemapJSON,
            'id': sitemap._id,
            'username': sitemap.username,
            'password': sitemap.password,
            'server': sitemap.server
        });
        $("#viewport").html(sitemapExportForm);
        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });

        jQuery('a.download').on('click', this.downloadResults);
        jQuery('a.cmd-action-run').on('click', this.initCasper);
        this.updateStatus();

        jQuery('input[name="username"]').on('input', function(e){
            controller.updateStatus();
        });
        jQuery('input[name="password"]').on('input', function(e){
            controller.updateStatus();
        });
        jQuery('input[name="server"]').on('input', function(e){
            controller.updateStatus();
        });

        return true;
    },
    showSitemaps: function () {
        if (arguments.length > 0 && arguments[0] !== false) {
            this.saveRecorder();
        }

        this.clearState();
        var controller = this;
        this.store.getAllSitemaps(function (sitemaps) {
            $sitemapListPanel = ich.SitemapList();
            sitemaps.forEach(function (sitemap) {
                $sitemap = ich.SitemapListItem(sitemap);
                $sitemap.data("sitemap", sitemap);
                $sitemapListPanel.find("tbody").append($sitemap);
            });
            $("#viewport").html($sitemapListPanel);
            controller.store.getLng(function (result) {
                controller.setupLanguage(result.lng);
            });

            if (!sitemaps.length) {
                controller.showNoItemsMessage('#sitemaps', 'createTemplateMessage');
                jQuery('#sitemaps table').addClass('hidden');
            }
        });
    },
    createSitemap: function (form) {
        var sitemap = new Sitemap({
            _id: $("#create-sitemap input[name=_id]").val(),
            startUrl: $("#create-sitemap input[name=startUrl]").val(),
            countOfStepsLimit: $("#countOfStepsLimit").val(),
            skippedValues: $("#create-sitemap input[name=skippedValues]").val(),
            selectors: [],
            crawlWeb: $("#create-sitemap input[name=crawlWeb]").is(":checked"),
            username: $("#username").val(),
            server: $("#server").val(),
            password: $("#password").val()
        });
        this.store.createSitemap(sitemap, function (sitemap) {
            this._editSitemap(sitemap, ['_root']);
        }.bind(this, sitemap));
    },
    importSitemap: function () {
        var sitemapJSON = $("[name=sitemapJSON]").val();
        var sitemap = new Sitemap();
        sitemap.importSitemap(sitemapJSON);
        this.store.createSitemap(sitemap, function (sitemap) {
            this._editSitemap(sitemap, ['_root']);
        }.bind(this, sitemap));
    },
    editSitemapMetadata: function (button) {
        var sitemap = this.state.currentSitemap;
        var $sitemapMetadataForm = ich.SitemapEditMetadata(sitemap);
        $("#viewport").html($sitemapMetadataForm);
        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });

        return true;
    },
    editSitemapMetadataSave: function (button) {
        var sitemap = this.state.currentSitemap;
        var data = {
            id: $("#edit-sitemap input[name=_id]").val(),
            startUrl: $("#edit-sitemap input[name=startUrl]").val(),
            countOfStepsLimit: $("#countOfStepsLimit").val(),
            skippedValues: $("#edit-sitemap input[name=skippedValues]").val(),
            crawlWeb: $("#edit-sitemap input[name=crawlWeb]").is(":checked"),
            username: $("#username").val(),
            server: $("#server").val(),
            password: $("#password").val()
        };

        var controller = this;

        // change data
        sitemap.startUrl = data.startUrl;
        sitemap.linkLimit = data.linkLimit;
        sitemap.skippedValues = data.skippedValues;
        sitemap.countOfStepsLimit = data.countOfStepsLimit;
        sitemap.crawlWeb = data.crawlWeb;
        sitemap.username = data.username;
        sitemap.server = data.server;
        sitemap.password = data.password;

        // just change sitemaps url
        if (data.id === sitemap._id) {
            controller.store.saveSitemap(sitemap, function (sitemap) {
                controller.showSitemapSelectorList();
            });
        }
        // id changed. we need to delete the old one and create a new one
        else {
            var newSitemap = new Sitemap(sitemap);
            var oldSitemap = sitemap;
            newSitemap._id = data.id;
            controller.store.createSitemap(newSitemap, function (newSitemap) {
                controller.store.deleteSitemap(oldSitemap, function () {
                    this.state.currentSitemap = newSitemap;
                    controller.showSitemapSelectorList();
                }.bind(this));
            }.bind(this));
        }
    },
    /**
     * Callback when sitemap edit button is clicked in sitemap grid
     */
    editSitemap: function (tr) {
        var sitemap = $(tr).data("sitemap");
        this._editSitemap(sitemap);
    },
    _editSitemap: function (sitemap) {
        this.setStateEditSitemap(sitemap);
        this.showRecorder();
    },
    showSitemapSelectorList: function () {
        this.saveRecorder();

        var sitemap = this.state.currentSitemap;

        var $selectorListPanel = ich.SelectorList();
        var selectors = sitemap.getSelectors();

        selectors.forEach(function (selector) {
            $selector = ich.SelectorListItem(selector);
            $selector.data("selector", selector);
            $selectorListPanel.find("tbody").append($selector);
        });
        $("#viewport").html($selectorListPanel);

        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });

        if (!selectors.length) {
            jQuery('#selector-tree table').addClass('hidden');
            controller.showNoItemsMessage('#selector-tree', 'createElement');
        }
        return true;
    },
    saveRecorder: function () {
        var newSitemap = jQuery('.recorder-sitemap').val();
        var formConfiguration = new Array();
        var delay = new Array();
        var countStep = new Array();

        jQuery('.recorder-item').each(function () {
            var id = jQuery('input:checkbox', this).data("item-id");

            formConfiguration.push({
                value: jQuery('input.column[data-item-id="' + id + '"]', this).val(),
                id: id
            });

            delay.push({
                value: jQuery('input.delay[data-item-id="' + id + '"]', this).val(),
                id: id
            });

            countStep.push({
                value: jQuery('input:checkbox', this).is(':checked'),
                id: id
            });

        });

        if (newSitemap != '' && newSitemap != undefined) {
            this.state.currentSitemap = new Sitemap(JSON.parse(jQuery('.recorder-sitemap').val()));
            this.store.createSitemap(this.state.currentSitemap);
        }

        if (formConfiguration.length) {
            this.state.currentSitemap.formConfiguration = formConfiguration;
            this.state.currentSitemap.delay = delay;
            this.state.currentSitemap.countStep = countStep;
            this.store.createSitemap(this.state.currentSitemap);
        }
    },
    showSitemapSelectorGraph: function () {
        var sitemap = this.state.currentSitemap;
        var $selectorGraphPanel = ich.SitemapSelectorGraph();
        $("#viewport").html($selectorGraphPanel);
        var graphDiv = $("#selector-graph")[0];
        var graph = new SelectorGraphv2(sitemap);
        graph.draw(graphDiv, $(document).width(), 200);
        return true;
    },
    showChildSelectors: function (tr) {
        var selector = $(tr).data('selector');
        var parentSelectors = this.state.editSitemapBreadcumbsSelectors;
        this.state.currentParentSelectorId = selector.id;
        parentSelectors.push(selector);

        this.showSitemapSelectorList();
    },
    treeNavigationshowSitemapSelectorList: function (button) {
        var parentSelectors = this.state.editSitemapBreadcumbsSelectors;
        var controller = this;
        $("#selector-tree .breadcrumb li a").each(function (i, parentSelectorButton) {
            if (parentSelectorButton === button) {
                parentSelectors.splice(i + 1);
                controller.state.currentParentSelectorId = parentSelectors[i].id;
            }
        });
        this.showSitemapSelectorList();
    },
    editSelector: function (button) {
        var selector = $(button).closest("tr").data('selector');
        this._editSelector(selector);
    },
    _editSelector: function (selector) {

        var sitemap = this.state.currentSitemap;
        var selectorIds = sitemap.getPossibleParentSelectorIds();

        var $editSelectorForm = ich.SelectorEdit({
            selector: selector,
            selectorIds: selectorIds,
            selectorTypes: [
                {
                    type: 'SelectorText',
                    title: 'Text'
                },
                {
                    type: 'SelectorLink',
                    title: 'Url'
                },
                {
                    type: 'SelectorCurrentUrl',
                    title: 'Current URL'
                },
                {
                    type: 'SelectorImage',
                    title: 'Image'
                },
                {
                    type: 'SelectorHTML',
                    title: 'HTML'
                }
            ],
            selectorFilters: [
                {
                    filter: 'ReduceWhiteSpaces',
                    title: 'Reduce white spaces'
                },
                {
                    filter: 'RemoveHTMLEntities',
                    title: 'Remove HTML entities'
                },
                {
                    filter: 'Country',
                    title: 'Select country'
                }
            ]
        });
        $("#viewport").html($editSelectorForm);
        // handle selects seperately
        $editSelectorForm.find("[name=type]").val(selector.type);

        if (selector.hasOwnProperty('filters') != false && selector.filters) {
            $editSelectorForm.find("[name=filters]").val(selector.filters);
            selector.filters.forEach(function (filter) {
                $editSelectorForm.find("#filters [value='" + filter + "']").attr("selected", "selected");
            });
        }

        jQuery(".chzn-select").chosen();

        jQuery('.select-type').on('change', function () {
            var request = {
                changeSelector: true
            };

            chrome.runtime.sendMessage(request);

            jQuery('#selector').val('');
            jQuery.trigger('click', 'button[action="select-selector"]');
        });

        this.state.currentSelector = selector;

        var controller = this;
        this.store.getLng(function (result) {
            controller.setupLanguage(result.lng);
        });

        setTimeout(function () {
            jQuery(".chzn-select").trigger("chosen:updated");

            if (jQuery('#filters_chosen .search-field input').val()) {
                jQuery('#filters_chosen .search-field input').val(i18n.t('selectOptions'));
            }
        }, 3000);
    },
    saveSelector: function (button) {
        var sitemap = this.state.currentSitemap;
        var selector = this.state.currentSelector;
        var newSelector = this.getCurrentlyEditedSelector();

        sitemap.updateSelector(selector, newSelector);

        this.store.saveSitemap(sitemap, function () {
            this.showSitemapSelectorList();
        }.bind(this));
    },
    /**
     * Get selector from selector editing form
     */
    getCurrentlyEditedSelector: function () {
        var id = $("#edit-selector [name=id]").val();
        var selectorsSelector = $("#edit-selector [name=selector]").val();
        var type = $("#edit-selector [name=type]").val();
        var filters = $("#edit-selector [name=filters]").val();
        var required = $("#edit-selector [name=required]").is(":checked");

        var newSelector = new Selector({
            id: id,
            selector: selectorsSelector,
            filters: filters,
            type: type,
            required: required
        });

        return newSelector;
    },
    /**
     * @returns {Sitemap|*} Cloned Sitemap with currently edited selector
     */
    getCurrentlyEditedSelectorSitemap: function () {
        var sitemap = this.state.currentSitemap.clone();
        var selector = sitemap.getSelectorById(this.state.currentSelector.id);
        var newSelector = this.getCurrentlyEditedSelector();

        sitemap.updateSelector(selector, newSelector);
        return sitemap;
    },
    cancelSelectorEditing: function (button) {
        this.showSitemapSelectorList();
    },
    addSelector: function () {

        var parentSelectorId = this.state.currentParentSelectorId;
        var sitemap = this.state.currentSitemap;

        var selector = new Selector({
            parentSelectors: [parentSelectorId],
            type: 'SelectorText',
            required: false
        });

        this._editSelector(selector, sitemap);
    },
    deleteSelector: function (button) {

        var sitemap = this.state.currentSitemap;
        var selector = $(button).closest("tr").data('selector');
        sitemap.deleteSelector(selector);

        this.store.saveSitemap(sitemap, function () {
            this.showSitemapSelectorList();
        }.bind(this));
    },
    deleteSitemap: function (button) {
        var sitemap = $(button).closest("tr").data("sitemap");
        var controller = this;
        this.store.deleteSitemap(sitemap, function () {
            controller.showSitemaps();
        });
    },
    scrapeSitemap: function () {
        var sitemap = this.state.currentSitemap;
        var request = {
            scrapeSitemap: true,
            sitemap: JSON.parse(JSON.stringify(sitemap))
        };
        chrome.runtime.sendMessage(request, function (response) {
            // sitemap scraped
        });
        return true;
    },
    sitemapListBrowseSitemapData: function (button) {
        var sitemap = $(button).closest("tr").data("sitemap");
        this.setStateEditSitemap(sitemap);
        this.browseSitemapData();
    },
    browseSitemapData: function () {
        var sitemap = this.state.currentSitemap;
        this.store.getSitemapData(sitemap, function (data) {

            var dataColumns = sitemap.getDataColumns();

            var dataPanel = ich.SitemapBrowseData({
                columns: dataColumns
            });
            $("#viewport").html(dataPanel);

            // display data
            // Doing this the long way so there aren't xss vulnerubilites 
            // while working with data or with the selector titles
            var $tbody = $("#sitemap-data tbody");
            data.forEach(function (row) {
                var $tr = $("<tr></tr>");
                dataColumns.forEach(function (column) {
                    var $td = $("<td></td>");
                    var cellData = row[column];
                    if (typeof cellData === 'object') {
                        cellData = JSON.stringify(cellData);
                    }
                    $td.text(cellData);
                    $tr.append($td);
                });
                $tbody.append($tr);
            });
        }.bind(this));

        return true;
    },
    showSitemapExportDataCsvPanel: function () {
        var sitemap = this.state.currentSitemap;
        var exportPanel = ich.SitemapExportDataCSV(sitemap);
        $("#viewport").html(exportPanel);

        // generate data
        $(".download-button").hide();
        this.store.getSitemapData(sitemap, function (data) {
            var blob = sitemap.getDataExportCsvBlob(data);
            $(".download-button a").attr("href", window.URL.createObjectURL(blob));
            $(".download-button a").attr("download", sitemap._id + ".csv");
            $(".download-button").show();
        }.bind(this));

        return true;
    },
    selectSelector: function (button) {
        var sitemap = this.getCurrentlyEditedSelectorSitemap();
        var selector = this.getCurrentlyEditedSelector();

        // run css selector through background page
        var request = {
            selectSelector: true,
            selectorId: selector.id,
            sitemap: JSON.parse(JSON.stringify(sitemap)),
            translation: i18n.t("doneSelecting"),
            translationParent: i18n.t("selectParent"),
            translationChild: i18n.t("selectChild")
        };

        chrome.runtime.sendMessage(request, function (response) {
            $("#edit-selector input[name=selector]").val(response.selector);
        });
    },
    selectSelectorParent: function (button) {
        var request = {
            selectSelectorParent: true
        };
        chrome.runtime.sendMessage(request);
    },
    selectSelectorChild: function (button) {
        var request = {
            selectSelectorChild: true
        };
        chrome.runtime.sendMessage(request);
    },
    previewSelector: function (button) {

        if ($(button).hasClass('active')) {
            var sitemap = this.getCurrentlyEditedSelectorSitemap();
            var selector = this.getCurrentlyEditedSelector();

            // run css selector through background page
            var request = {
                previewSelector: true,
                sitemap: JSON.parse(JSON.stringify(sitemap)),
                selectorId: selector.id
            };
            chrome.runtime.sendMessage(request);
        }
        else {
            var request = {
                cancelPreviewSelector: true
            };
            chrome.runtime.sendMessage(request);
        }
    },
    previewSelectorFromSelectorTree: function (button) {

        if (!$(button).hasClass('active')) {
            $(button).addClass('active');

            var sitemap = this.state.currentSitemap;
            var selector = $(button).closest("tr").data('selector');

            // run css selector through background page
            var request = {
                previewSelector: true,
                sitemap: JSON.parse(JSON.stringify(sitemap)),
                selectorId: selector.id
            };
            chrome.runtime.sendMessage(request);
        }
        else {
            $(button).removeClass('active');
            var request = {
                cancelPreviewSelector: true
            };
            chrome.runtime.sendMessage(request);
        }
    },
    previewSelectorDataFromSelectorTree: function (button) {
        var sitemap = this.state.currentSitemap;
        var selector = $(button).closest("tr").data('selector');
        this.previewSelectorData(sitemap, selector.id);
    },
    previewSelectorDataFromSelectorEditing: function () {
        var sitemap = this.state.currentSitemap.clone();
        var selector = sitemap.getSelectorById(this.state.currentSelector.id);
        var newSelector = this.getCurrentlyEditedSelector();
        sitemap.updateSelector(selector, newSelector);
        this.previewSelectorData(sitemap, newSelector.id);
    },
    getConfigFormCsv: function () {
        var formConfiguration = this.state.currentSitemap.formConfiguration;
        var data = [];

        for (var i = 0; i < formConfiguration.length; i++) {
            data.push(formConfiguration[i].value);
        }

        return data.join(';') + "\n";
    },
    previewSelectorData: function (sitemap, selectorId) {
        var request = {
            previewSelectorData: true,
            sitemap: JSON.parse(JSON.stringify(sitemap)),
            selectorId: selectorId
        };
        var controller = this;
        chrome.runtime.sendMessage(request, function (response) {

            if (response.length === 0) {
                return
            }
            var dataColumns = Object.keys(response[0]);

            console.log(dataColumns);

            var $dataPreviewPanel = ich.DataPreview({
                columns: dataColumns
            });
            $("#viewport").append($dataPreviewPanel);
            $dataPreviewPanel.modal('show');
            // display data
            // Doing this the long way so there aren't xss vulnerubilites
            // while working with data or with the selector titles
            var $tbody = $("tbody", $dataPreviewPanel);
            response.forEach(function (row) {
                var $tr = $("<tr></tr>");
                dataColumns.forEach(function (column) {
                    var $td = $("<td></td>");
                    var cellData = row[column];
                    if (typeof cellData === 'object') {
                        cellData = JSON.stringify(cellData);
                    }
                    $td.text(cellData);
                    $tr.append($td);
                });
                $tbody.append($tr);
            });

            var windowHeight = $(window).height();

            $(".data-preview-modal .modal-body").height(windowHeight - 130);

            // remove modal from dom after it is closed
            $dataPreviewPanel.on("hidden.bs.modal", function () {
                $(this).remove();
            });

            controller.store.getLng(function (result) {
                controller.setupLanguage(result.lng);
            });
        });
    }
};
