var SelectorList = function (selectors) {

	if(selectors === undefined) {
		return;
	}

	for(var i = 0;i<selectors.length;i++) {
		this.push(selectors[i]);
	}
};

SelectorList.prototype = new Array;

SelectorList.prototype.push = function(selector) {

	if(!this.hasSelector(selector.id)) {
		if(!(selector instanceof Selector)) {
			selector = new Selector(selector);
		}
		Array.prototype.push.call(this, selector);
	}
};

SelectorList.prototype.hasSelector = function(selectorId) {

	if(selectorId instanceof Object) {
		selectorId = selectorId.id;
	}

	for (var i = 0; i < this.length; i++) {
		if(this[i].id === selectorId) {
			return true;
		}
	}
	return false;
};

/**
 * Returns all selectors or recursively find and return all child selectors of a parent selector.
 * @param parentSelectorId
 * @returns {Array}
 */
SelectorList.prototype.getAllSelectors = function(parentSelectorId) {

	if(!parentSelectorId) {
		return this;
	}

	return [];
};

/**
 * Returns only selectors that are directly under a parent
 * @param parentSelectorId
 * @returns {Array}
 */
SelectorList.prototype.getSelectors = function(parentSelectorId) {
	var resultSelectors = new SelectorList();
	this.forEach(function(selector) {
			resultSelectors.push(selector);
	});
	return resultSelectors;
};

SelectorList.prototype.clone = function() {
	var resultList = new SelectorList();
	this.forEach(function(selector){
		resultList.push(selector);
	});
	return resultList;
};

SelectorList.prototype.fullClone = function() {
	var resultList = new SelectorList();
	this.forEach(function(selector){
		resultList.push(JSON.parse(JSON.stringify(selector)));
	});
	return resultList;
};

SelectorList.prototype.concat = function() {
	var resultList = this.clone();
	for(var i in arguments) {
		arguments[i].forEach(function(selector) {
			resultList.push(selector);
		}.bind(this));
	}
	return resultList;
};

SelectorList.prototype.getSelector = function(selectorId) {
	for (var i = 0; i < this.length; i++) {
		var selector = this[i];
		if (selector.id === selectorId) {
			return selector;
		}
	}
};

/**
 * Returns all child selectors of a selector which can be used within one page.
 * @param parentSelectorId
 */
SelectorList.prototype.getSinglePageAllChildSelectors = function(parentSelectorId) {

	var resultList = new SelectorList();
	var childSelectors = this.getSelectors();
	childSelectors.forEach(function (childSelector) {
		resultList.push(childSelector);
	}.bind(this));
	return resultList;
};

/**
 * When serializing to JSON convert to an array
 * @returns {Array}
 */
SelectorList.prototype.toJSON = function() {
	var result = [];
	this.forEach(function(selector) {
		result.push(selector);
	});
	return result;
};

SelectorList.prototype.getSelectorById = function (selectorId) {
	for (var i = 0; i < this.length; i++) {
		var selector = this[i];
		if (selector.id === selectorId) {
			return selector;
		}
	}
}