if (typeof(EventTypes) == "undefined") {
    EventTypes = {};
}

EventTypes.OpenUrl = 0;
EventTypes.Click = 1;
EventTypes.Change = 2;
EventTypes.Comment = 3;
EventTypes.Submit = 4;
EventTypes.CheckPageTitle = 5;
EventTypes.CheckPageLocation = 6;
EventTypes.CheckTextPresent = 7;
EventTypes.CheckValue = 8;
EventTypes.CheckValueContains = 9;
EventTypes.CheckText = 10;
EventTypes.CheckHref = 11;
EventTypes.CheckEnabled = 12;
EventTypes.CheckDisabled = 13;
EventTypes.CheckSelectValue = 14;
EventTypes.CheckSelectOptions = 15;
EventTypes.CheckImageSrc = 16;
EventTypes.PageLoad = 17;
EventTypes.ScreenShot = 18;
EventTypes.MouseDown = 19;
EventTypes.MouseUp = 20;
EventTypes.MouseDrag = 21;
EventTypes.MouseDrop = 22;
EventTypes.KeyPress = 23;

if (typeof(FunctionTypes) == "undefined") {
    FunctionTypes = {};
}

FunctionTypes[EventTypes.OpenUrl] = "openUrl";
FunctionTypes[EventTypes.Change] = "change";
FunctionTypes[EventTypes.Click] = "click";
FunctionTypes[EventTypes.Comment] = "comment";
FunctionTypes[EventTypes.Submit] = "submit";
FunctionTypes[EventTypes.CheckPageTitle] = "checkPageTitle";
FunctionTypes[EventTypes.CheckPageLocation] = "checkPageLocation";
FunctionTypes[EventTypes.CheckTextPresent] = "checkTextPresent";
FunctionTypes[EventTypes.CheckValue] = "checkValue";
FunctionTypes[EventTypes.CheckText] = "checkText";
FunctionTypes[EventTypes.CheckHref] = "checkHref";
FunctionTypes[EventTypes.CheckEnabled] = "checkEnabled";
FunctionTypes[EventTypes.CheckDisabled] = "checkDisabled";
FunctionTypes[EventTypes.CheckSelectValue] = "checkSelectValue";
FunctionTypes[EventTypes.CheckSelectOptions] = "checkSelectOptions";
FunctionTypes[EventTypes.CheckImageSrc] = "checkImageSrc";
FunctionTypes[EventTypes.PageLoad] = "pageLoad";
FunctionTypes[EventTypes.ScreenShot] = "screenShot";
FunctionTypes[EventTypes.MouseDown] = "mouseDown";
FunctionTypes[EventTypes.MouseUp] = "mouseUp";
FunctionTypes[EventTypes.MouseDrag] = "mousedrag";
FunctionTypes[EventTypes.KeyPress] = "keypress";

function NavigationElement(data) {
    this.prepareData(data);
}

NavigationElement.prototype.prepareData = function(data){
    for(var key in data){
        this[key] = data[key];
        if(Array.isArray(data[key])){
            this.prepareData(data[key]);
        }
    }
}

NavigationElement.prototype.getInputText = function(formConfiguration, data, navigationStep) {
    var inputText = '';

    if (typeof data == "undefined") {
        return this.text;
    }

    for (var key in formConfiguration) {
        if (formConfiguration[key].id == navigationStep) {
            inputText = data[formConfiguration[key].value];
        }
    }

    if (!data) {
        return this.text;
    }

    return inputText;
}

NavigationElement.prototype.getSelector = function() {
    var selector = '';

    if (this.info == undefined) {
        return null;
    }

    if (this.info.form != undefined) {
        selector += 'form';
        if (this.info.form.id != undefined && this.info.form.id != '') {
            selector += '#' + this.info.form.id;
        } else if (this.info.form.class != undefined && this.info.form.class != '') {
            selector += '.' + this.info.form.class;
        }
        selector += ' ';
    }

    selector += this.info.selector;

    return selector;
}

NavigationElement.prototype.getMethodName = function() {
    if (FunctionTypes.hasOwnProperty(this.type)) {
        return FunctionTypes[this.type];
    }

    return null;
}

module.exports = NavigationElement;
