var WebSocketServer = require('websocket').server;
var http = require('http');
var fs = require('fs');
var crypto = require('crypto');
var express = require('express');
var bodyParser = require('body-parser');
var multer = require('multer');
var cookieParser = require('cookie-parser');
var cors = require('cors');
var iconv = require('iconv-lite');

var serverPort = 16002;
if(process.argv.indexOf("-server-port") != -1){
    serverPort = process.argv[process.argv.indexOf("-server-port") + 1];
}

var upload = multer();
var app = express();

var corsOptions = {
    credentials: true,
    origin: true
};

var PATH_ROOT = require('path').dirname(require.main.filename);
PATH_ROOT = PATH_ROOT.split('\\');
PATH_ROOT.pop();
PATH_ROOT = PATH_ROOT.join('/');

app.use(cors(corsOptions));
app.use(cookieParser());
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({extended: true}));

app.get('/', function (req, res) {
    try {
        res.sendStatus(200);
    } catch (e) {
        res.sendStatus(500);
    }
});

app.post('/status', function (req, res, next) {
    try {
        if (!req.body || !authentication(req.body.username, req.body.password)) {
            res.sendStatus(403);
            return;
        }

        res.sendStatus(200);
    } catch (e) {
        res.sendStatus(500);
    }
});

app.post('/init', function (req, res, next) {
    try {
        if (!req.body || !authentication(req.body.username, req.body.password)) {
            res.sendStatus(403);
            return;
        }

        if (!createPlugin(req.body.configuration)) {
            res.sendStatus(500);
            return;
        }

        res.sendStatus(200);
    } catch (e) {
        res.sendStatus(500);
    }
});

app.post('/upload-config-form', upload.single('file'), function (req, res, next) {
    try {
        if (!req.body || !authentication(req.body.username, req.body.password)) {
            res.sendStatus(403);
            return;
        }

        var configuration = JSON.parse(req.body.configuration);
        var plugin = configuration._id;
        var newPath = PATH_ROOT + '/plugins/' + plugin + '/config-form-cp1250.csv'

        fs.writeFileSync(newPath, req.file.buffer);

        fs.createReadStream(newPath)
            .pipe(iconv.decodeStream('cp1250'))
            .pipe(iconv.encodeStream('utf8'))
            .pipe(fs.createWriteStream(PATH_ROOT + '/plugins/' + plugin + '/config-form.csv'));

        res.sendStatus(200);
    } catch (e) {
        res.sendStatus(500);
    }
});

app.post('/download', function (req, res) {
    try {
        if (!authentication(req.body.username, req.body.password)) {
            res.sendStatus(403);
            return;
        }

        var file = PATH_ROOT + '/plugins/' + req.query.plugin + '/' + req.query.file;

        if (!fs.existsSync(file)) {
            res.sendStatus(500);
            return;
        }

        if (req.query.file == 'config-form.csv') {
            file = PATH_ROOT + '/plugins/' + req.query.plugin + '/config-form-cp1250.csv';
        }

        res.download(file);
    } catch (e) {
        res.sendStatus(500);
    }
});

app.get('/screenshot', function (req, res) {
    try {
        var options = {
            root: PATH_ROOT + '/server/',
            dotfiles: 'deny',
            headers: {
                'x-timestamp': Date.now(),
                'x-sent': true
            }
        };

        var fileName = 'screenshot.png';
        res.sendFile(fileName, options);
    } catch (e) {
        res.sendStatus(500);
    }
});

var server = app.listen(serverPort, function () {
    var host = server.address().address;
    var port = server.address().port;
});

var BUF_LENGTH = 64 * 1024;
var _buff = new Buffer(BUF_LENGTH);

wsServer = new WebSocketServer({
    httpServer: server,
    autoAcceptConnections: false
});

wsServer.on('request', function (request) {
    var connection = request.accept('echo-protocol', request.origin);
    terminal = {};

    connection.on('message', function (message) {
        var configuration = JSON.parse(message.utf8Data);
        var args = [];

        args.push('--ssl-protocol=any');

        if (!authentication(configuration.username, configuration.password)) {
            connection.sendUTF('Access denied');
            connection.close();
        }

        if (configuration.proxyServer) {
            args.push('--proxy=' + configuration.proxyServer);
        }

        if (configuration.proxyServer && configuration.proxyUsername && configuration.proxyPassword) {
            args.push('--proxy-auth=' + configuration.proxyUsername + ':' + configuration.proxyPassword);
        }

        args.push(PATH_ROOT + "/scraper.js");
        args.push("--load-plugin=" + configuration.id);

        var isWin = /^win/.test(process.platform);
        console.log(isWin);
        if (isWin) {
            terminal = require('child_process').spawn('casperjs.bat', args);
        } else {
            terminal = require('child_process').spawn('casperjs', args);
        }

        terminal.stdout.on('data', function (data) {
            connection.sendUTF(data);
        });

        terminal.on('exit', function (code) {
            connection.close(1000, 'Scraping ended');
            terminal.kill();
            file = PATH_ROOT + '/plugins/' + configuration.id + '/results.csv';

            if (fs.existsSync(file)){
                fs.createReadStream(file)
                    .pipe(iconv.decodeStream('utf8'))
                    .pipe(iconv.encodeStream('cp1250'))
                    .pipe(fs.createWriteStream(PATH_ROOT + '/plugins/' + configuration.id + '/results-cp1250.csv'));
            }

        });
    });


    connection.on('close', function (reasonCode, description) {
        terminal.kill();
    });
});

function createPlugin(configuration) {
    if (!configuration) {
        return false;
    }

    configuration = JSON.parse(configuration);
    if (!fs.existsSync(PATH_ROOT + '/plugins/' + configuration._id)) {
        fs.mkdirSync(PATH_ROOT + '/plugins/' + configuration._id);
    }

    fs.writeFileSync(PATH_ROOT + '/plugins/' + configuration._id + '/config.json', new Buffer(JSON.stringify(configuration)));

    var data = [];

    if (configuration.hasOwnProperty('formConfiguration')) {
        for (var i = 0; i < configuration.formConfiguration.length; i++) {
            if (!configuration.formConfiguration[i].value) {
                continue;
            }
            data.push(configuration.formConfiguration[i].value);
        }
    }

        fs.writeFileSync(PATH_ROOT + '/plugins/' + configuration._id + '/config-form.csv', data.join(';'));

    var newPath = PATH_ROOT + '/plugins/' + configuration._id + '/config-form-cp1250.csv'

    fs.createReadStream(PATH_ROOT + '/plugins/' + configuration._id + '/config-form.csv')
        .pipe(iconv.decodeStream('cp1250'))
        .pipe(iconv.encodeStream('utf8'))
        .pipe(fs.createWriteStream(newPath));

    return true;
}

function getUsers() {
    var users = [];
    fs.readFileSync('./users.csv').toString().split(/\r?\n/).forEach(function (line) {
        var user = line.split(';');
        users.push({username: user[0], password: user[1]});
    });

    return users;
}

function authentication(username, password) {
    var index;
    var users = getUsers();

    for (index in users) {
        user = users[index];

        if (password && crypto.createHash('md5').update(password).digest("hex") == user.password
            && username && user.username == username) {
            return true;
        }
    }

    return false;
}